==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/equipment/bauble/RingOfCorrectionItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import net.minecraft.class_1268;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_239;
import net.minecraft.class_2680;
import net.minecraft.class_3965;
import vazkii.botania.api.item.SortableTool;
import vazkii.botania.common.item.equipment.tool.ToolCommons;

public class RingOfCorrectionItem extends BaubleItem {

	public RingOfCorrectionItem(class_1793 props) {
		super(props);
	}

	@Override
	public void onWornTick(class_1799 stack, class_1309 entity) {
		if (entity.field_6002.field_9236 || !(entity instanceof class_1657 player)) {
			return;
		}

		class_1799 currentStack = player.method_6047();
		if (currentStack.method_7960() || !(currentStack.method_7909() instanceof SortableTool tool)
				|| !player.field_6252) {
			return;
		}

		class_3965 pos = ToolCommons.raytraceFromEntity(player, 4.5F, false);
		if (pos.method_17783() != class_239.class_240.field_1332) {
			return;
		}
		class_2680 state = entity.field_6002.method_8320(pos.method_17777());

		class_1799 bestTool = currentStack;
		int bestToolPriority = currentStack.method_7924(state) > 1.0F ? tool.getSortingPriority(currentStack, state) : -1;
		int bestSlot = -1;

		for (int i = 0; i < player.method_31548().method_5439(); i++) {
			class_1799 stackInSlot = player.method_31548().method_5438(i);
			if (!stackInSlot.method_7960() && stackInSlot.method_7909() instanceof SortableTool toolInSlot && stackInSlot != currentStack) {
				if (stackInSlot.method_7924(state) > 1.0F) {
					int priority = toolInSlot.getSortingPriority(stackInSlot, state);
					if (priority > bestToolPriority) {
						bestTool = stackInSlot;
						bestToolPriority = priority;
						bestSlot = i;
					}
				}
			}
		}

		if (bestSlot != -1) {
			player.method_6122(class_1268.field_5808, bestTool);
			player.method_31548().method_5447(bestSlot, currentStack);
		}
	}
}

==> Botania-1.20.1-451-FABRIC-20260131.092931-41-sources.jar: vazkii/botania/common/item/equipment/bauble/RingOfCorrectionItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import net.minecraft.class_1268;
import net.minecraft.class_1309;
import net.minecraft.class_1799;
import net.minecraft.class_239;
import net.minecraft.class_2680;
import net.minecraft.class_3222;
import net.minecraft.class_3965;
import vazkii.botania.api.item.SortableTool;
import vazkii.botania.common.item.equipment.tool.ToolCommons;
import vazkii.botania.mixin.ServerPlayerGameModeAccessor;

public class RingOfCorrectionItem extends BaubleItem {

	public RingOfCorrectionItem(class_1793 props) {
		super(props);
	}

	@Override
	public void onWornTick(class_1799 stack, class_1309 entity) {
		if (entity.method_37908().field_9236 || !(entity instanceof class_3222 player)) {
			return;
		}

		class_1799 currentStack = player.method_6047();
		if (currentStack.method_7960() || !(currentStack.method_7909() instanceof SortableTool tool)
				|| !((ServerPlayerGameModeAccessor) player.field_13974).botania_isDestroyingBlock()) {
			return;
		}

		class_3965 pos = ToolCommons.raytraceFromEntity(player, 4.5F, false);
		if (pos.method_17783() != class_239.class_240.field_1332) {
			return;
		}
		class_2680 state = entity.method_37908().method_8320(pos.method_17777());

		class_1799 bestTool = currentStack;
		int bestToolPriority = currentStack.method_7924(state) > 1.0F ? tool.getSortingPriority(currentStack, state) : -1;
		int bestSlot = -1;

		for (int i = 0; i < player.method_31548().method_5439(); i++) {
			class_1799 stackInSlot = player.method_31548().method_5438(i);
			if (!stackInSlot.method_7960() && stackInSlot.method_7909() instanceof SortableTool toolInSlot && stackInSlot != currentStack) {
				if (stackInSlot.method_7924(state) > 1.0F) {
					int priority = toolInSlot.getSortingPriority(stackInSlot, state);
					if (priority > bestToolPriority) {
						bestTool = stackInSlot;
						bestToolPriority = priority;
						bestSlot = i;
					}
				}
			}
		}

		if (bestSlot != -1) {
			player.method_6122(class_1268.field_5808, bestTool);
			player.method_31548().method_5447(bestSlot, currentStack);
		}
	}
}
